import { useBreakpoints } from '@vueuse/core'
import type { Breakpoints } from '~/constants'
import { BreakpointsEnum, BreakpointsMap } from '~/constants'

let globalScreenRef: ComputedRef<BreakpointsEnum>
let globalWidthRef: ComputedRef<number>
let globalRealWidthRef: ComputedRef<number>

export interface BreakpointCallback {
  screen: ComputedRef<BreakpointsEnum>
  width: ComputedRef<number>
  realWidth: ComputedRef<number>
  breakpointsMap: Breakpoints
  // vueuse breakpoints
  greater(k: BreakpointsEnum): Ref<boolean>
  greaterOrEqual: (k: BreakpointsEnum) => Ref<boolean>
  smaller(k: BreakpointsEnum): Ref<boolean>
  smallerOrEqual(k: BreakpointsEnum): Ref<boolean>
  between(a: BreakpointsEnum, b: BreakpointsEnum): Ref<boolean>
  isGreater(k: BreakpointsEnum): boolean
  isGreaterOrEqual(k: BreakpointsEnum): boolean
  isSmaller(k: BreakpointsEnum): boolean
  isSmallerOrEqual(k: BreakpointsEnum): boolean
  isInBetween(a: BreakpointsEnum, b: BreakpointsEnum): boolean
  current(): ComputedRef<string[]>
}

export function useBreakpoint() {
  const breakpoints = useBreakpoints(BreakpointsMap)

  const screenRef = computed(() => unref(globalScreenRef))

  return {
    ...breakpoints,
    screenRef,
  }
}

export function createBreakpointListen(fn?: (p: BreakpointCallback) => void) {
  const breakpoints = useBreakpoint()
  const screenRef = ref<BreakpointsEnum>(BreakpointsEnum.XL)
  let realWidthRef = $ref(window.innerWidth)

  const {
    greater,
    greaterOrEqual,
    isGreater,
    isGreaterOrEqual,
    between,
    isInBetween,
    smaller,
    smallerOrEqual,
    isSmaller,
    isSmallerOrEqual,
    current,
  } = breakpoints

  function getWindowWidth() {
    realWidthRef = document.body.clientWidth
    screenRef.value = (breakpoints.isSmallerOrEqual(BreakpointsEnum.XS)
      ? BreakpointsEnum.XS
      : breakpoints.isInBetween(BreakpointsEnum.XS, BreakpointsEnum.SM)
        ? BreakpointsEnum.SM
        : breakpoints.isInBetween(BreakpointsEnum.SM, BreakpointsEnum.MD)
          ? BreakpointsEnum.MD
          : breakpoints.isInBetween(BreakpointsEnum.MD, BreakpointsEnum.LG)
            ? BreakpointsEnum.LG
            : breakpoints.isInBetween(BreakpointsEnum.LG, BreakpointsEnum.XL)
              ? BreakpointsEnum.XL
              : breakpoints.isInBetween(BreakpointsEnum.XL, BreakpointsEnum.XXL) ? BreakpointsEnum.XXL : BreakpointsEnum.XL)
  }

  function resizeFn() {
    fn?.({
      screen: globalScreenRef,
      width: globalWidthRef,
      realWidth: globalRealWidthRef,
      breakpointsMap: BreakpointsMap,
      greater,
      greaterOrEqual,
      isGreater,
      isGreaterOrEqual,
      between,
      isInBetween,
      smaller,
      smallerOrEqual,
      isSmaller,
      isSmallerOrEqual,
      current,
    })
  }

  useEventListener(window, 'resize', () => {
    getWindowWidth()
    resizeFn()
  })

  getWindowWidth()

  globalScreenRef = computed(() => unref(screenRef))
  globalWidthRef = computed((): number => BreakpointsMap[unref(screenRef)] as number)
  globalRealWidthRef = computed((): number => unref(realWidthRef))

  resizeFn()
}
